var fs = require('fs');
var toSource = require('tosource');
var UglifyJS = require("uglify-js");
var Ctx = require("./context");
var schemaCompiler = require("./schemaCompiler");
var cssCompiler = require("./cssCompiler");
var jsCompiler = require("./jsCompiler");
var path = require("path");
var minimatch = require("minimatch");

var Compiler = function (cfg) {

    var filterSchema = function(schema, deps) {
        var copy = {};
        for (var blk in schema) {
            if (checkInclude(blk) && checkExclude(blk)) {
                copy[blk] = schema[blk];
                for (var d in deps[blk]) {
                    var n = deps[blk][d];
                    if (!copy[n]) copy[n] = schema[n];
                }
            }
        }
        return copy;
    };

    var checkInclude = function(blk) {
        if (!cfg.tahoe.client.include) return true;
        for (var ind in cfg.tahoe.client.include) {
            if (minimatch(blk, cfg.tahoe.client.include[ind])) {
                return true;
            }
        }
        return false;
    };

    var checkExclude = function(blk) {
        if (!cfg.tahoe.client.exclude) return true;
        for (var ind in cfg.tahoe.client.exclude) {
            if (minimatch(blk, cfg.tahoe.client.exclude[ind])) {
                return false;
            }
        }
        return true;
    };


    this.compile = function (cb) {
        var data;
        if (cfg.tahoe.params) eval("data = " + fs.readFileSync(cfg.tahoe.params));
        var d = 0;
        var done = function(err) {
            if (err) return cb(err);
            if (++d == 3 && cb) return cb(null);
        };
        cssCompiler(cfg, data, function(css){
            var dir = cfg.tahoe.publicDir;
            fs.writeFileSync(path.join(dir, cfg.tahoe.name + ".css"), css.css);
            for (var t in css.themes) {
                fs.writeFileSync(path.join(dir, t + ".css"), css.themes[t]);
            }
            done();
        });
        schemaCompiler(cfg, function(err, ctx) {
            if (err) {
                return done(err);
            }
            var cont = fs.readFileSync(__dirname + '/context.js') + '';

            var source = toSource(ctx.schema);
            var data = cont + "Ctx.schema = " + source;
            if (cfg.tahoe.minify) data =  UglifyJS.minify(data, {fromString: true}).code;
            fs.writeFileSync(path.join(cfg.tahoe.destDir, cfg.tahoe.name + "-schema.js"), data);

            if (cfg.tahoe.client) {
                source = toSource(filterSchema(ctx.schema, ctx.deps));
                data = cont + "Ctx.schema = " + source;
                if (cfg.tahoe.minify) data = UglifyJS.minify(data, {fromString: true}).code;
                fs.writeFileSync(path.join(cfg.tahoe.publicDir, cfg.tahoe.name + "-schema-client.js"), data);
            }
            return done(null);
        });
        jsCompiler(cfg, function(js) {
            if (cfg.tahoe.minify) {
                js =  UglifyJS.minify(js, {fromString: true}).code;
            }
            fs.writeFileSync(path.join(cfg.tahoe.publicDir, cfg.tahoe.name + "-client.js"), js);
            done();
        });
    };
    this.getContext = function(cb) {
        schemaCompiler(cfg, function(ctx) {
            Ctx.schema = ctx.schema;
            cb(Ctx);
        });
    };
};

module.exports = Compiler;



