var fs = require("fs");
var recursive = require("./recursive");
var path = require("path");
//var tags = require("./tags");
var sax = require("sax"),
    strict = true, // set to false for html-mode
    parser = sax.parser(strict);

var Ctx = function() {
    this.seq = 0;
    this.schema = {};
    this.prefixes = {};
    this.fileName = "";
    var self = this;
    this.deps = [];

    this.startNewFile = function(fileName) {
        self.fileName = fileName.replace(/\\/g, "/");
        var res = path.parse(self.fileName);
        self.name = res.dir;
        if (self.name == "") {
            self.name = res.name;
        } else if (res.name !== "index") {
            self.name += "/"+res.name;
        }
    };
};

var tags = {
    block: require("./tags/Block"),
    root: require("./tags/Root"),
    def: require("./tags/Def"),
    parent: require("./tags/Parent"),
    include: require("./tags/Include"),
    html: require("./tags/Html"),
    script: require("./tags/Script"),
    for: require("./tags/For"),
    if: require("./tags/If"),
    import: require("./tags/Import"),
    func: require("./tags/Func"),
    set: require("./tags/Set"),

    getTag: function(ctx, node, parent) {
        var m = node.name.split(":");
        var pr = m[0];
        var fn = m[1];
        var t;
        if (tags[fn] && typeof tags[fn] === 'function' ) {
            t = tags[fn];
        } else if (pr && fn) {
            t = tags.func;
        } else {
            t = tags.html;
        }
        var inst = new t();
        inst.init(ctx, node, parent);
        return inst;
    }

};

var stack = [];

parser.onerror = function (e) {
};

parser.ontext = function (t) {
    var inst = stack[stack.length-1];
    t = t.trim();
    if (t.length > 0 && inst['text'] && typeof inst['text'] === 'function' ) inst.text(t);
};

parser.onopentag = function (node) {
    var inst = tags.getTag(ctx, node, stack[stack.length-1]);
    stack.push(inst);
    inst.start();
};

parser.onclosetag = function (name) {
    var inst = stack.pop();
    if (inst && inst['end'] && typeof inst['end'] === 'function' ) inst.end();
    if (inst.deps && inst.deps.length > 0) {
        ctx.deps[inst.name] = inst.deps;
    }

};

parser.onattribute = function (attr) {};

parser.onend = function () {};


var parseFile = function(dir, file) {
    var xml = fs.readFileSync(file, "UTF-8");
    var fileName = path.relative(dir, file);
    ctx.startNewFile(fileName);
    parser.write(xml).close();
};

var ctx = new Ctx();

var buildSchema = function (cfg, cb) {
    var fileList = [];
    cfg.tahoe.sourceDirs.forEach(function (d) {
        recursive(d).forEach(function(f) {
            fileList.push({dir:d, file:f});
        });
    });

    for (var ind in fileList) {
        var f = fileList[ind];
        if (f.file.indexOf(".xml", f.length - 4) == -1) continue;
        try {
            parseFile(f.dir, f.file);
        } catch (e) {
            return cb(new Error("File \"" + f.file + "\" - " + e.message), null);
        }
    }
    cb(null, ctx);
};

module.exports = buildSchema;