var vm = require("vm");
var tags = {};
var exp = require("./expression");
var hash = require("string-hash");
module.exports = tags;

var Body = function() {
    this.s = "";
    this.write = function(str) {
        this.s += str + "\n";
    }
};

var parent = false;
var schema = {macros:{}, blocks:{}, files:{}};

var prefixes = {};
var curHash;

tags.startNewFile = function(fileName) {
    fileName = fileName.replace(/\\/g, "/");
    curHash = fileName;
    prefixes = {};
//    schema.files[fileName] = curHash;
};

tags.getSchema = function() {
    return schema;
};

var evalFunc = function(str) {
    var sandbox = {};
    try {
        vm.runInNewContext("var a = function(ctx) {" + str + "}", sandbox);
    } catch (e) {
        console.error("Error in " + str);
    }
    return sandbox.a;
};

var evalObj = function(str) {
    var sandbox = {};
    try {
        vm.runInNewContext("var a = function(ctx) { return " + str + "}", sandbox);
    } catch (e) {
        console.error("Error in " + str);
    }
    return sandbox.a;
};

var getParentBlock = function(tag) {
    if (!tag) return;
    if (tag.el) return tag;
    else return getParentBlock(tag.parent);
};

var replacePrefix = function (name, pref) {
    if (pref) {
        return pref + ":" + name;
    }
    var n = name;
    for (var p in prefixes) {
        var ph = p + ":";
        if (n.indexOf(ph) != -1) {
            n = n.replace(ph, prefixes[p] + ":");
            break;
        }
    }

    if (n == name) {
        n = curHash + ":" + name;
    }
    return n;
};

tags.import = function(n, parent) {
    this.node = n;

    this.start = function() {
        var file = this.node.attributes['file'];
        file = file.replace(/\\/g, "/");
        var prefix = this.node.attributes['prefix'];
        prefixes[prefix] = file;
    };
};

tags.macro = function(n, parent) {
    this.node = n;
    this.name = n.attributes['name'];
    this.el = {};
    this.body = new Body();
    this.parent = parent;

    this.start = function() {
        if (this.node.attributes['parent']) this.el.parent = this.node.attributes['parent'];
    };

    this.text = function(text) {
        this.body.write('ctx.write('+exp.resolveStr(text, "'")+');');
    };

    this.end = function() {
        if (!this.node.attributes['parent']) {
            this.el.body = evalFunc(this.body.s);
        } else if (!this.el.def && this.body.s.indexOf("ctx.def(") == -1) {
            this.el.def = evalFunc(this.body.s);
        }
        schema.blocks[this.name] = this.el;
    }
};

tags.html = function(n, parent) {
    this.node = n;
    this.body = parent.body;
    this.parent = parent;

    this.start = function() {
        var attrs = "";
        for (var attr in this.node.attributes) {
            var a = this.node.attributes[attr].trim();
            attrs += " " + attr + "='" + a + "'";
        }
        if (this.node.isSelfClosing) {
            this.body.write("ctx.write(" + exp.resolveStr("<"+this.node.name+attrs+"/>", "'") + ");");
        } else {
            this.body.write("ctx.write(" + exp.resolveStr("<"+this.node.name+attrs+">", "'") + ");");
        }
    };

    this.text = function(text) {
        this.body.write("ctx.write("+exp.resolveStr(text, "'")+");");
    };

    this.end = function() {
        if (!this.node.isSelfClosing)
            this.body.write('ctx.write("</'+this.node.name+'>");');
    }
};

tags.script = function(n, parent) {
    this.node = n;
    this.body = parent.body;
    this.start = function() {};
    this.text = function(text) {
        this.body.write(text);
    };
};

tags.func = function(n, parent) {
    this.node = n;
    this.body = parent.body;
    this.parent = parent;
    this.name = "";
    this.el = {};

    this.start = function() {
        this.name = this.node.name.split(":")[1];
        var attrs = "{";
        for (var attr in this.node.attributes) {
            var a = this.node.attributes[attr].trim();
            attrs += attr + ":" + exp.resolveStr(a, "'") + ",";
        }
        attrs += "}";
        if (!this.node.isSelfClosing) {
            this.el.parent = exp.hasExp(this.name) ? evalFunc("return " + exp.resolveStr("${name}", "'")) : this.name;
            var pTag = getParentBlock(parent);
            if (!pTag.anCount) pTag.anCount = 0;
            this.name = "__" + pTag.name+"__"+pTag.anCount++;
            this.body.write("ctx.callMacro('" + this.name + "', " + attrs + ");");
            this.body = new Body();
        } else {
            this.body.write("ctx.callMacro('" + this.name + "', " + attrs + ");");
        }
    };

    this.text = function(text) {
        this.body.write('ctx.write('+exp.resolveStr(text, "'")+');');
    };

    this.end = function() {
        if (this.node.isSelfClosing) return;
        if (!this.el.def) {
            this.el.def = evalFunc(this.body.s);
        }
        schema.blocks[this.name] = this.el;
    }

};

tags.def = function(n, parent) {
    this.node = n;
    this.body = new Body();
    var name = "def";
    this.parent = parent;

    this.start = function() {
        if (this.node.attributes.name) {
            name += "_" + this.node.attributes.name;
        }
        parent.body.write("ctx.def('" + name + "');");
    };

    this.text = function(text) {
        this.body.write("ctx.write("+exp.resolveStr(text, "'")+");");
    };
    this.end = function() {
        getParentBlock(parent).el[name] = evalFunc(this.body.s);
    };
};

tags.parent = function(n, parent) {
    this.node = n;
    this.body = parent.body;
    this.start = function() {
        this.body.write("ctx.parent();");
    };
};

tags.if = function(n, parent) {
    this.node = n;
    this.body = parent.body;
    this.parent = parent;

    this.start = function() {
        this.body.write("if (" + exp.resolveStr(this.node.attributes['exp']) + ") {");
    };

    this.text = function(text) {
        this.body.write('ctx.write('+exp.resolveStr(text, "'")+');');
    };

    this.end = function() {
        this.body.write("};");
    };

};

tags.for = function(n, parent) {
    this.node = n;
    this.body = parent.body;
    this.parent = parent;
    this.start = function() {
        var v = this.node.attributes['var'];
        var i = this.node.attributes['in'];
        var s = exp.resolveStr(i);
        this.body.write("for (var " + v + " in " + s + ") {");
        this.body.write("ctx.attrs['" + v + "']=" + s+"["+v+"]" + ";");
    };

    this.text = function(text) {
        this.body.write('ctx.write('+exp.resolveStr(text, "'")+');');
    };

    this.end = function() {
        this.body.write("};");
    };

};

tags.block = function(n, parent) {
    this.node = n;
    this.name = n.attributes['name'];
    this.el = {};
    this.parent = parent;

    this.start = function () {
        this.body = new Body();
        this.name = curHash + ":" + this.name;
        if (this.node.attributes['parent']) {
            this.el.parent = this.node.attributes['parent'];
            this.el.parent = replacePrefix(this.el.parent);
        }
        var attrs = "{";
        for (var attr in this.node.attributes) {
            if (attr == "name" || attr == "parent") continue;
            var a = this.node.attributes[attr].trim();
            attrs += attr + ":" + exp.resolveStr(a, "'") + ",";
        }
        attrs += "}";
        this.el.data = evalObj(attrs);
    };

    this.text = function(text) {
        this.body.write('ctx.write('+exp.resolveStr(text, "'")+');');
    };

    this.end = function() {
        if (this.node.isSelfClosing) return;
        if (!this.node.attributes['parent']) {
            this.el.body = evalFunc(this.body.s);
        } else if (!this.el.def && this.body.s.indexOf("ctx.def(") == -1) {
            this.el.def = evalFunc(this.body.s);
        }
        schema.blocks[this.name] = this.el;
    }
};

tags.include = function(n, parent) {
    this.node = n;
    this.name = n.attributes['name'];
    this.el = {};
    this.parent = parent;

    this.start = function() {
        this.body = parent.body;

        if (this.node.attributes['file']) {
            this.el.file = this.node.attributes['file'];
        }

        var attrs = "{";
        for (var attr in this.node.attributes) {
            if (attr == "name" || attr == "parent") continue;
            var a = this.node.attributes[attr].trim();
            attrs += attr + ":" + exp.resolveStr(a, "'") + ",";
        }
        attrs += "}";
        this.name = replacePrefix(this.name, this.el.file);
        if (!this.node.isSelfClosing) {
            this.el.parent = exp.hasExp(this.name) ? evalFunc("return " + exp.resolveStr("${name}", "'")) : this.name;
            var pTag = getParentBlock(parent);
            if (!pTag.anCount) pTag.anCount = 0;
            this.name = "__" + pTag.name+"__"+pTag.anCount++;
            this.body.write("ctx.callBlock('" + this.name + "', " + attrs + ");");
            this.body = new Body();
        } else {
            this.name = exp.resolveStr(this.name, "'");
            this.body.write("ctx.callBlock(" + this.name + ", " + attrs + ");");
        }
    };

    this.text = function(text) {
        this.body.write('ctx.write('+exp.resolveStr(text, "'")+');');
    };

    this.end = function() {
        if (this.node.isSelfClosing) return;
        if (!this.el.def) {
            this.el.def = evalFunc(this.body.s);
        }
        schema.blocks[this.name] = this.el;
    }

};

tags.root = function(n) {
    this.start = function() {};
};

tags.getTag = function(node, parent) {
    var fn = node.name.split(":")[1];
    var inst;
    if (tags[fn] && typeof tags[fn] === 'function' ) {
        inst = tags[fn];
    } else if (fn) {
        inst = tags.func;
    } else {
        inst = tags.html;
    }
    return new inst(node, parent);
};