var Compiler = require("../lib/compiler.js");

var cw = {};

module.exports = {
    setUp: function(callback) {
        if (cw.ctx) {
            this.ctx = cw.ctx;
            return callback();
        }

        var cfg = {
            "tahoe": {
                "sourceDirs": ["test/blocks"],
                "dest": "res"
            }
        };

        var compiler = new Compiler(cfg);
        var self = this;
        compiler.getContext(function(ctx) {
            cw.ctx = ctx;
            self.ctx = ctx;
            callback();
        });
    },
    "var": function(test) {
        var s = this.ctx.renderSync("var", "var", {x:"value"});
        test.equal(s, "value\n");
        test.done();
    },
    "parent_test": function(test) {
        var s = this.ctx.renderSync("var", "parent_test", {x:"value"});
        test.equal(s, "value\n");
        test.done();
    },
    "anon_def_test": function(test) {
        var s = this.ctx.renderSync("var", "anon_def_test", {x:"value"});
        test.equal(s, "value\n");
        test.done();
    },
    "call_parent": function(test) {
        var s = this.ctx.renderSync("var", "call_parent", {});
        test.equal(s, "parent\n");
        test.done();
    },
    "context": function(test) {
        var s = this.ctx.renderSync("var", "context", {x:"hello"});
        var ex = "0-open hello\n1-open undefined\n2-open hello\nin hello\n2-close hello\n1-close undefined\n0-close hello\n";
        test.equal(s, ex);
        test.done();
    },
    "anon/anon": function(test) {
        var s = this.ctx.renderSync("anon/anon", "B", {x:"hello"});
        var ex = "hello\n";
//        console.log(s);
        test.equal(s, ex);
        test.done();
    },
    "anon/cascade": function(test) {
        var s = this.ctx.renderSync("anon/cascade", "D", {x:"hello"});
        var ex = "d\nc\nb\na\nhello\na\nb\nc\nd\n";
//        console.log(s);
        test.equal(s, ex);
        test.done();
    },
    "anon/cascade-named": function(test) {
        var s = this.ctx.renderSync("anon/cascade-named", "D", {x:"hello"});
        var ex = "d\nc\nb\na\nhello\na\nb\nc\nd\n";
//        console.log(s);
        test.equal(s, ex);
        test.done();
    },
    "inheritance/simple": function(test) {
        var s = this.ctx.renderSync("inheritance/simple", "B", {});
        var ex = "hello\n";
//        console.log(s);
        test.equal(s, ex);
        test.done();
    },
    "inheritance/multi": function(test) {
        var s = this.ctx.renderSync("inheritance/multi", "C", {});
        var ex = "hello from C\n";
//        console.log(s);
        test.equal(s, ex);
        test.done();
    },
    "inheritance/named-defs": function(test) {
        var s = this.ctx.renderSync("inheritance/named-defs", "C", {});
        var ex = "a in C\nb in C\n";
//        console.log(s);
        test.equal(s, ex);
        test.done();
    },
    "inheritance/cascade": function(test) {
        var s = this.ctx.renderSync("inheritance/cascade", "C", {});
        var ex = "a1\nb1\nb in C\nb2\na2\n";
//        console.log(s);
        test.equal(s, ex);
        test.done();
    },
    "inheritance/default-defs": function(test) {
        var s = this.ctx.renderSync("inheritance/default-defs", "C", {});
        var ex = "a definition\n";
//        console.log(s);
        test.equal(s, ex);
        test.done();
    }


};